// USB MIDI->CV/DIN MIDI box
// by Staffan Melin
// License: GNU General Public License v3.0
//
// Note: This is for 5V CV
//
// Inspiration:
// https://github.com/felis/USB_Host_Shield_2.0/blob/master/examples/USBH_MIDI/USB_MIDI_converter/USB_MIDI_converter.ino
// http://henrysbench.capnfatz.com/henrys-bench/arduino-output-devices/arduino-mcp4725-digital-to-analog-converter-tutorial/

// Libraries:
// USB Host Shield Library 2.0
// Adafruit MPC4725
//
// Outputs:
// CV: DAC Out (1V/oct)
// GATE: Arduino D9
// GND: DAC Gnd
// MIDI (Arduino D3)



// INCLUDES AND DEFINES

// #define DEBUG

// MIDI host
#include <usbh_midi.h>
#include <usbhub.h>
#include <SPI.h>

// DAC
#include <Wire.h>
#include <Adafruit_MCP4725.h>
 // The I2C Address. Download och run the I2C Scanner to look up the address
#define DAC_ADDRESS 0x62
// Connect DAC out to this to test (and define DEBUG)
#define DAC_TEST A0

// Gate
#define GATE 9
bool gated = false;
byte lastNote = 0;

// MIDI out
#include <SoftwareSerial.h>




// VARIABLES

// USB host
USB Usb;
USBH_MIDI  Midi(&Usb);

// DAC
Adafruit_MCP4725 dac;

// MIDI
SoftwareSerial mySerial(2, 3); // RX, TX
#define MIDI_NOTE_ON 144
#define MIDI_NOTE_OFF 128
#define MIDI_CHANNEL 0

// functions
void MIDI_poll();
void doDelay(uint32_t t1, uint32_t t2, uint32_t delayTime);



void setup()
{
  
  // setup SoftSerial for MIDI control
  mySerial.begin(31250);
  delay(100);
  
  #ifdef DEBUG
  Serial.begin(38400);
  #endif
  
  if (Usb.Init() == -1) {
    while (1);
  }
  delay(200);

  dac.begin(DAC_ADDRESS);

  pinMode(GATE, OUTPUT);
}



void loop()
{
  Usb.Task();
  
  uint32_t t1 = (uint32_t)micros();

  if ( Usb.getUsbTaskState() == USB_STATE_RUNNING )
  {
    MIDI_poll();
  }
  
  // delay (1ms)
  doDelay(t1, (uint32_t)micros(), 1000);
  
}


// We only look for and act upon NOTE ON and NOTE OFF messages
void MIDI_poll()
{
  uint8_t outBuf[ 3 ];
  uint8_t size;
  byte value;

  uint32_t dac_value;
  int adcValueRead = 0;
  float voltageRead = 0;

  do {
    if ((size = Midi.RecvData(outBuf)) > 0 ) {

      if (size < 3){
        for (byte i; i < size; i++) {
          Serial.print(outBuf[i],3 );      
          Serial.println(); 
        }     
      }

      if (size == 3) {

          //MIDI Output
          //_MIDI_SERIAL_PORT.write(outBuf, size);

        value = outBuf[0];

        // NOTE ON?
        
        if (value == MIDI_NOTE_ON) {
          lastNote = outBuf[1];

          dac_value = midiToCV(lastNote); // NOTE
          dac.setVoltage(dac_value, false);

          digitalWrite(GATE, LOW);
          digitalWrite(GATE, HIGH);
          gated = true;

          // MIDI: send
          midiNote(MIDI_NOTE_ON + MIDI_CHANNEL, lastNote, outBuf[2]);

          #ifdef DEBUG
          Serial.print("NOTE ON: MIDI2CV: ");
          Serial.print(dac_value);
          delay(10);
          adcValueRead = analogRead(DAC_TEST);
          voltageRead = adcValueRead * 4;
          Serial.print("\tArduino ADC Value: ");
          Serial.print(adcValueRead);
          Serial.print("\t ADC*4: ");      
          Serial.print(voltageRead,3);      
          Serial.println();
          #endif

        // NOTE OFF?
        
        } else if (value = MIDI_NOTE_OFF) {
          
          #ifdef DEBUG
          Serial.println("NOTE OFF");
          #endif

          // only silence note if we are still playing the same note
          if (lastNote == outBuf[1]) {
            digitalWrite(GATE, LOW);
            gated = false;
          }

          // MIDI: send
          midiNote(MIDI_NOTE_OFF + MIDI_CHANNEL, outBuf[1], outBuf[2]);

        // not NOTE ON nor NOTE OFF
        } else {
          
          #ifdef DEBUG
          for (byte i; i < size; i++) {
            Serial.print(outBuf[i],3 );      
            Serial.println(); 
          }
          #endif

        }
      }
    }
  } while (size > 0);
}



// Delay time (max 16383 us)
void doDelay(uint32_t t1, uint32_t t2, uint32_t delayTime)
{
  uint32_t t3;

  if ( t1 > t2 ) {
    t3 = (0xFFFFFFFF - t1 + t2);
  } else {
    t3 = t2 - t1;
  }

  if ( t3 < delayTime ) {
    delayMicroseconds(delayTime - t3);
  }
}



// convert midi pitch to 12 bit control voltage
// from: https://gist.github.com/toddtreece/5449fdb479ddadd0ed5b
int midiToCV(int pitch) {
  return (int) round(((float)pitch - 34.27538387) * 69.400442636);
}


//  Play a MIDI note

void midiNote(int aMidiCommand, int aMidiPitch, int aMidiVelocity) {
  mySerial.write(aMidiCommand);
  mySerial.write(aMidiPitch);
  mySerial.write(aMidiVelocity);
}
